library example.basic_auth.server;

import 'dart:async';
import 'dart:io';

import 'package:jaguar/jaguar.dart';
import 'package:jaguar_session/jaguar_session.dart';
import 'package:jaguar_serializer/serializer.dart';
import 'package:jaguar_json/jaguar_json.dart';

import '../../common/models/models.dart';
import '../../common/books.dart';

CookieSession sessionManager() => new CookieSession();

/// This route group contains login and logout authentication routes
@Api()
class AuthRoutes extends Object with JsonRoutes {
  JsonRepo get repo => jsonRepo;

  @Post(path: '/login/:username/:pwd')
  Future<Response> login(Context ctx) async {
    String username = ctx.pathParams['username'];
    String pwd = ctx.pathParams['pwd'];
    if (username == null || pwd == null) {
      throw new Response(null, statusCode: HttpStatus.UNAUTHORIZED);
    }

    if (username != 'jaguar' || pwd != 'awesome') {
      throw new Response(null, statusCode: HttpStatus.UNAUTHORIZED);
    }

    Response resp = new Response(null, statusCode: 200);
    CookieSession sesMan = sessionManager();
    //Set session information
    await sesMan.updateSession({'username': 'jaguar'});
    //Update response with session information
    return await sesMan.writeResponse(ctx.req, resp);
  }

  @Post(path: '/logout')
  Future<Response> logout(Context ctx) async {
    Response resp = new Response(null, statusCode: 200);
    CookieSession sesMan = sessionManager();
    //Clear session information on logout
    await sesMan.deleteSession();
    //Update response
    return await sesMan.writeResponse(ctx.req, resp);
  }
}

/// Collection of routes that need authorization
@Api(path: '/book')
class StudentRoutes extends Object with JsonRoutes {
  JsonRepo get repo => jsonRepo;

  @Get(path: '/all')
  Future<Response<String>> getAllBooks(Context ctx) async {
    CookieSession sesMan = sessionManager();
    //Parse session information
    await sesMan.parseRequest(ctx.req);

    //Authorize
    String username = sesMan.getInValue('username');
    if (username is! String || username != 'jaguar') {
      throw new Response(null, statusCode: HttpStatus.UNAUTHORIZED);
    }

    return toJson(books.values);
  }
}

@Api(path: '/api')
class LibraryApi {
  @IncludeApi()
  final AuthRoutes auth = new AuthRoutes();

  @IncludeApi()
  final StudentRoutes student = new StudentRoutes();
}

server() async {
  final jaguar = new Jaguar();
  jaguar.addApiReflected(new LibraryApi());

  await jaguar.serve();
}
